package extension

import (
	"testing"

	pb "git.perx.ru/perxis/perxis-go/proto/extensions"
	"github.com/brianvoe/gofakeit/v6"
	"github.com/stretchr/testify/require"
)

func TestActionToProto(t *testing.T) {
	t.Run("Filled struct", func(t *testing.T) {
		var pbAction *pb.Action
		err := gofakeit.Struct(&pbAction)
		require.NoError(t, err)
		
		action := ActionFromPB(pbAction)
		result := ActionToPB(action)
		require.Equal(t, pbAction, result)
	})

	t.Run("Action is nil", func(t *testing.T) {
		var pbAction *pb.Action
		action := ActionFromPB(pbAction)
		result := ActionToPB(action)
		require.Equal(t, pbAction, result)
	})
}

func TestActionToMap(t *testing.T) {
	t.Run("Filled struct", func(t *testing.T) {
		var action *Action
		err := gofakeit.Struct(&action)
		require.NoError(t, err)
		dict := ActionToMap(action)
		result, err := ActionFromMap(dict)
		require.NoError(t, err)
		require.Equal(t, action, result)
	})

	t.Run("Action is nil", func(t *testing.T) {
		var action *Action
		dict := ActionToMap(action)
		result, err := ActionFromMap(dict)
		require.NoError(t, err)
		require.Equal(t, &Action{}, result)
	})
}

func TestActionRequestToProto(t *testing.T) {
	t.Run("Filled struct", func(t *testing.T) {
		var pbAction *pb.ActionRequest
		err := gofakeit.Struct(&pbAction)
		require.NoError(t, err)
		
		action := ActionRequestFromPB(pbAction)
		result := ActionRequestToPB(action)
		require.Equal(t, pbAction, result)
	})

	t.Run("Action is nil", func(t *testing.T) {
		var pbAction *pb.ActionRequest
		action := ActionRequestFromPB(pbAction)
		result := ActionRequestToPB(action)
		require.Equal(t, pbAction, result)
	})
}

func TestActionResponseToProto(t *testing.T) {
	t.Run("Filled struct", func(t *testing.T) {
		var pbAction *pb.ActionResponse
		err := gofakeit.Struct(&pbAction)
		require.NoError(t, err)
	
		action := ActionResponseFromPB(pbAction)
		result := ActionResponseToPB(action)
		require.Equal(t, pbAction, result)
	})
	
	t.Run("Action is nil", func(t *testing.T) {
		var pbAction *pb.ActionResponse
		action := ActionResponseFromPB(pbAction)
		result := ActionResponseToPB(action)
		require.Equal(t, pbAction, result)
	})
}
